@@ -17,7 +17,6 @@
 use anyhow::bail;
 use std::fmt;
 use wasmer::{imports, wat2wasm, Function, Instance, Module, Store, TypedFunction};
-use wasmer_compiler_cranelift::Cranelift;
 
 // First we need to create an error type that we'll use to signal the end of execution.
 #[derive(Debug, Clone, Copy)]
@@ -51,10 +50,7 @@ fn main() -> anyhow::Result<()> {
     )?;
 
     // Create a Store.
-    // Note that we don't need to specify the engine/compiler if we want to use
-    // the default provided by Wasmer.
-    // You can use `Store::default()` for that.
-    let mut store = Store::new(Cranelift::default());
+    let mut store = Store::default();
 
     println!("Compiling module...");
     // Let's compile the Wasm module.
