@@ -6,7 +6,6 @@
 
 #![deny(missing_docs, unused_extern_crates)]
 #![warn(unused_import_braces)]
-#![cfg_attr(feature = "std", deny(unstable_features))]
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(feature = "cargo-clippy", allow(clippy::new_without_default))]
 #![cfg_attr(
@@ -21,6 +20,7 @@
         clippy::use_self
     )
 )]
+#![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
 
 #[cfg(all(feature = "std", feature = "core"))]
 compile_error!(
@@ -62,6 +62,7 @@ mod libcalls;
 mod memory;
 mod module;
 mod serialize;
+mod stack;
 mod store_id;
 mod table;
 mod trapcode;
@@ -125,11 +126,10 @@ pub use crate::compilation::function::{
     Functions,
 };
 pub use crate::compilation::module::CompileModuleInfo;
-pub use crate::compilation::sourceloc::SourceLoc;
 pub use crate::compilation::symbols::{Symbol, SymbolRegistry};
-pub use crate::compilation::trap::TrapInformation;
 pub use crate::compilation::unwind::CompiledFunctionUnwindInfo;
 
+pub use crate::stack::{FrameInfo, SourceLoc, TrapInformation};
 pub use crate::store_id::StoreId;
 
 /// Offset in bytes from the beginning of the function.
@@ -169,6 +169,11 @@ mod native {
         const WASM_TYPE: Type;
     }
 
+    impl NativeWasmType for u32 {
+        const WASM_TYPE: Type = Type::I32;
+        type Abi = Self;
+    }
+
     impl NativeWasmType for i32 {
         const WASM_TYPE: Type = Type::I32;
         type Abi = Self;
@@ -179,6 +184,11 @@ mod native {
         type Abi = Self;
     }
 
+    impl NativeWasmType for u64 {
+        const WASM_TYPE: Type = Type::I64;
+        type Abi = Self;
+    }
+
     impl NativeWasmType for f32 {
         const WASM_TYPE: Type = Type::F32;
         type Abi = Self;
