@@ -3,7 +3,7 @@
 
 //! An `Instance` contains all the runtime state used by execution of
 //! a WebAssembly module (except its callstack and register state). An
-//! `InstanceHandle` is a wrapper around `Instance` that manages
+//! `VMInstance` is a wrapper around `Instance` that manages
 //! how it is allocated and deallocated.
 
 mod allocator;
@@ -19,9 +19,9 @@ use crate::vmcontext::{
     VMFunctionImport, VMFunctionKind, VMGlobalDefinition, VMGlobalImport, VMMemoryDefinition,
     VMMemoryImport, VMSharedSignatureIndex, VMTableDefinition, VMTableImport, VMTrampoline,
 };
-use crate::LinearMemory;
 use crate::{FunctionBodyPtr, MaybeInstanceOwned, TrapHandlerFn, VMFunctionBody};
-use crate::{VMFuncRef, VMFunction, VMGlobal, VMMemory, VMTable};
+use crate::{LinearMemory, NotifyLocation};
+use crate::{VMConfig, VMFuncRef, VMFunction, VMGlobal, VMMemory, VMTable};
 pub use allocator::InstanceAllocator;
 use memoffset::offset_of;
 use more_asserts::assert_lt;
@@ -33,8 +33,7 @@ use std::fmt;
 use std::mem;
 use std::ptr::{self, NonNull};
 use std::slice;
-use std::sync::{Arc, Mutex};
-use std::thread::{current, park, park_timeout, Thread};
+use std::sync::Arc;
 use wasmer_types::entity::{packed_option::ReservedValue, BoxedSlice, EntityRef, PrimaryMap};
 use wasmer_types::{
     DataIndex, DataInitializer, ElemIndex, ExportIndex, FunctionIndex, GlobalIndex, GlobalInit,
@@ -42,20 +41,6 @@ use wasmer_types::{
     MemoryIndex, ModuleInfo, Pages, SignatureIndex, TableIndex, TableInitializer, VMOffsets,
 };
 
-#[derive(Hash, Eq, PartialEq, Clone, Copy)]
-struct NotifyLocation {
-    memory_index: u32,
-    address: u32,
-}
-
-struct NotifyWaiter {
-    thread: Thread,
-    notified: bool,
-}
-struct NotifyMap {
-    map: HashMap<NotifyLocation, Vec<NotifyWaiter>>,
-}
-
 /// A WebAssembly instance.
 ///
 /// The type is dynamically-sized. Indeed, the `vmctx` field can
@@ -105,9 +90,6 @@ pub(crate) struct Instance {
     /// will point to elements here for functions imported by this instance.
     imported_funcrefs: BoxedSlice<FunctionIndex, NonNull<VMCallerCheckedAnyfunc>>,
 
-    /// The Hasmap with the Notify for the Notify/wait opcodes
-    conditions: Arc<Mutex<NotifyMap>>,
-
     /// Additional context used by compiled WebAssembly code. This
     /// field is last, and represents a dynamically-sized array that
     /// extends beyond the nominal end of the struct (similar to a
@@ -276,6 +258,31 @@ impl Instance {
         }
     }
 
+    /// Get a locally defined or imported memory.
+    fn get_vmmemory_mut(&mut self, index: MemoryIndex) -> &mut VMMemory {
+        if let Some(local_index) = self.module.local_memory_index(index) {
+            unsafe {
+                self.memories
+                    .get_mut(local_index)
+                    .unwrap()
+                    .get_mut(self.context.as_mut().unwrap())
+            }
+        } else {
+            let import = self.imported_memory(index);
+            unsafe { import.handle.get_mut(self.context.as_mut().unwrap()) }
+        }
+    }
+
+    /// Get a locally defined memory as mutable.
+    fn get_local_vmmemory_mut(&mut self, local_index: LocalMemoryIndex) -> &mut VMMemory {
+        unsafe {
+            self.memories
+                .get_mut(local_index)
+                .unwrap()
+                .get_mut(self.context.as_mut().unwrap())
+        }
+    }
+
     /// Return the indexed `VMGlobalDefinition`.
     fn global(&self, index: LocalGlobalIndex) -> VMGlobalDefinition {
         unsafe { self.global_ptr(index).as_ref().clone() }
@@ -319,6 +326,7 @@ impl Instance {
     /// Invoke the WebAssembly start function of the instance, if one is present.
     fn invoke_start_function(
         &self,
+        config: &VMConfig,
         trap_handler: Option<*const TrapHandlerFn<'static>>,
     ) -> Result<(), Trap> {
         let start_index = match self.module.start_function {
@@ -349,7 +357,7 @@ impl Instance {
 
         // Make the call.
         unsafe {
-            catch_traps(trap_handler, || {
+            catch_traps(trap_handler, config, || {
                 mem::transmute::<*const VMFunctionBody, unsafe extern "C" fn(VMFunctionContext)>(
                     callee_address,
                 )(callee_vmctx)
@@ -797,51 +805,19 @@ impl Instance {
         }
     }
 
-    // To implement Wait / Notify, a HasMap, behind a mutex, will be used
-    // to track the address of waiter. The key of the hashmap is based on the memory
-    // and waiter threads are "park"'d (with or without timeout)
-    // Notify will wake the waiters by simply "unpark" the thread
-    // as the Thread info is stored on the HashMap
-    // once unparked, the waiter thread will remove it's mark on the HashMap
-    // timeout / awake is tracked with a boolean in the HashMap
-    // because `park_timeout` doesn't gives any information on why it returns
-    fn do_wait(&mut self, index: u32, dst: u32, timeout: i64) -> u32 {
-        // fetch the notifier
-        let key = NotifyLocation {
-            memory_index: index,
-            address: dst,
-        };
-        let mut conds = self.conditions.lock().unwrap();
-        let v = conds.map.entry(key).or_insert_with(Vec::new);
-        v.push(NotifyWaiter {
-            thread: current(),
-            notified: false,
-        });
-        drop(conds);
-        if timeout < 0 {
-            park();
+    fn memory_wait(memory: &mut VMMemory, dst: u32, timeout: i64) -> Result<u32, Trap> {
+        let location = NotifyLocation { address: dst };
+        let timeout = if timeout < 0 {
+            None
         } else {
-            park_timeout(std::time::Duration::from_nanos(timeout as u64));
+            Some(std::time::Duration::from_nanos(timeout as u64))
+        };
+        let waiter = memory.do_wait(location, timeout);
+        if waiter.is_err() {
+            // ret is None if there is more than 2^32 waiter in queue or some other error
+            return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
         }
-        let mut conds = self.conditions.lock().unwrap();
-        let v = conds.map.get_mut(&key).unwrap();
-        let id = current().id();
-        let mut ret = 0;
-        v.retain(|cond| {
-            if cond.thread.id() == id {
-                ret = if cond.notified { 0 } else { 2 };
-                false
-            } else {
-                true
-            }
-        });
-        if v.is_empty() {
-            conds.map.remove(&key);
-        }
-        if conds.map.len() > 1 << 32 {
-            ret = 0xffff;
-        }
-        ret
+        Ok(waiter.unwrap())
     }
 
     /// Perform an Atomic.Wait32
@@ -861,11 +837,8 @@ impl Instance {
 
         if let Ok(mut ret) = ret {
             if ret == 0 {
-                ret = self.do_wait(memory_index.as_u32(), dst, timeout);
-            }
-            if ret == 0xffff {
-                // ret is 0xffff if there is more than 2^32 waiter in queue
-                return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
+                let memory = self.get_local_vmmemory_mut(memory_index);
+                ret = Self::memory_wait(memory, dst, timeout)?;
             }
             Ok(ret)
         } else {
@@ -888,14 +861,10 @@ impl Instance {
         //}
 
         let ret = unsafe { memory32_atomic_check32(memory, dst, val) };
-
         if let Ok(mut ret) = ret {
             if ret == 0 {
-                ret = self.do_wait(memory_index.as_u32(), dst, timeout);
-            }
-            if ret == 0xffff {
-                // ret is 0xffff if there is more than 2^32 waiter in queue
-                return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
+                let memory = self.get_vmmemory_mut(memory_index);
+                ret = Self::memory_wait(memory, dst, timeout)?;
             }
             Ok(ret)
         } else {
@@ -920,11 +889,8 @@ impl Instance {
 
         if let Ok(mut ret) = ret {
             if ret == 0 {
-                ret = self.do_wait(memory_index.as_u32(), dst, timeout);
-            }
-            if ret == 0xffff {
-                // ret is 0xffff if there is more than 2^32 waiter in queue
-                return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
+                let memory = self.get_local_vmmemory_mut(memory_index);
+                ret = Self::memory_wait(memory, dst, timeout)?;
             }
             Ok(ret)
         } else {
@@ -950,11 +916,8 @@ impl Instance {
 
         if let Ok(mut ret) = ret {
             if ret == 0 {
-                ret = self.do_wait(memory_index.as_u32(), dst, timeout);
-            }
-            if ret == 0xffff {
-                // ret is 0xffff if there is more than 2^32 waiter in queue
-                return Err(Trap::lib(TrapCode::TableAccessOutOfBounds));
+                let memory = self.get_vmmemory_mut(memory_index);
+                ret = Self::memory_wait(memory, dst, timeout)?;
             }
             Ok(ret)
         } else {
@@ -962,21 +925,6 @@ impl Instance {
         }
     }
 
-    fn do_notify(&mut self, key: NotifyLocation, count: u32) -> Result<u32, Trap> {
-        let mut conds = self.conditions.lock().unwrap();
-        let mut cnt = 0u32;
-        if let Some(v) = conds.map.get_mut(&key) {
-            for waiter in v {
-                if cnt < count {
-                    waiter.notified = true; // mark as was waiked up
-                    waiter.thread.unpark(); // wakeup!
-                    cnt += 1;
-                }
-            }
-        }
-        Ok(cnt)
-    }
-
     /// Perform an Atomic.Notify
     pub(crate) fn local_memory_notify(
         &mut self,
@@ -984,17 +932,10 @@ impl Instance {
         dst: u32,
         count: u32,
     ) -> Result<u32, Trap> {
-        //let memory = self.memory(memory_index);
-        //if ! memory.shared {
-        // We should trap according to spec, but official test rely on not trapping...
-        //}
-
+        let memory = self.get_local_vmmemory_mut(memory_index);
         // fetch the notifier
-        let key = NotifyLocation {
-            memory_index: memory_index.as_u32(),
-            address: dst,
-        };
-        self.do_notify(key, count)
+        let location = NotifyLocation { address: dst };
+        Ok(memory.do_notify(location, count))
     }
 
     /// Perform an Atomic.Notify
@@ -1004,18 +945,10 @@ impl Instance {
         dst: u32,
         count: u32,
     ) -> Result<u32, Trap> {
-        //let import = self.imported_memory(memory_index);
-        //let memory = unsafe { import.definition.as_ref() };
-        //if ! memory.shared {
-        // We should trap according to spec, but official test rely on not trapping...
-        //}
-
+        let memory = self.get_vmmemory_mut(memory_index);
         // fetch the notifier
-        let key = NotifyLocation {
-            memory_index: memory_index.as_u32(),
-            address: dst,
-        };
-        self.do_notify(key, count)
+        let location = NotifyLocation { address: dst };
+        Ok(memory.do_notify(location, count))
     }
 }
 
@@ -1023,8 +956,8 @@ impl Instance {
 ///
 /// This is more or less a public facade of the private `Instance`,
 /// providing useful higher-level API.
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub struct InstanceHandle {
+#[derive(Debug, Eq, PartialEq)]
+pub struct VMInstance {
     /// The layout of `Instance` (which can vary).
     instance_layout: Layout,
 
@@ -1040,8 +973,24 @@ pub struct InstanceHandle {
     instance: NonNull<Instance>,
 }
 
-impl InstanceHandle {
-    /// Create a new `InstanceHandle` pointing at a new [`InstanceRef`].
+/// VMInstance are created with an InstanceAllocator
+/// and it will "consume" the memory
+/// So the Drop here actualy free it (else it would be leaked)
+impl Drop for VMInstance {
+    fn drop(&mut self) {
+        let instance_ptr = self.instance.as_ptr();
+
+        unsafe {
+            // Need to drop all the actual Instance members
+            instance_ptr.drop_in_place();
+            // And then free the memory allocated for the Instance itself
+            std::alloc::dealloc(instance_ptr as *mut u8, self.instance_layout);
+        }
+    }
+}
+
+impl VMInstance {
+    /// Create a new `VMInstance` pointing at a new [`Instance`].
     ///
     /// # Safety
     ///
@@ -1051,7 +1000,7 @@ impl InstanceHandle {
     /// method is a low-overhead way of saying “this is an extremely unsafe type
     /// to work with”.
     ///
-    /// Extreme care must be taken when working with `InstanceHandle` and it's
+    /// Extreme care must be taken when working with `VMInstance` and it's
     /// recommended to have relatively intimate knowledge of how it works
     /// internally if you'd like to do so. If possible it's recommended to use
     /// the `wasmer` crate API rather than this type since that is vetted for
@@ -1109,12 +1058,9 @@ impl InstanceHandle {
                 funcrefs,
                 imported_funcrefs,
                 vmctx: VMContext {},
-                conditions: Arc::new(Mutex::new(NotifyMap {
-                    map: HashMap::new(),
-                })),
             };
 
-            let mut instance_handle = allocator.write_instance(instance);
+            let mut instance_handle = allocator.into_vminstance(instance);
 
             // Set the funcrefs after we've built the instance
             {
@@ -1198,6 +1144,7 @@ impl InstanceHandle {
     /// Only safe to call immediately after instantiation.
     pub unsafe fn finish_instantiation(
         &mut self,
+        config: &VMConfig,
         trap_handler: Option<*const TrapHandlerFn<'static>>,
         data_initializers: &[DataInitializer<'_>],
     ) -> Result<(), Trap> {
@@ -1209,7 +1156,7 @@ impl InstanceHandle {
 
         // The WebAssembly spec specifies that the start function is
         // invoked automatically at instantiation time.
-        instance.invoke_start_function(trap_handler)?;
+        instance.invoke_start_function(config, trap_handler)?;
         Ok(())
     }
 
