@@ -1,3 +1,5 @@
+use std::task::Waker;
+
 use super::*;
 use crate::syscalls::*;
 
@@ -20,6 +22,7 @@ pub(crate) fn thread_sleep_internal<M: MemorySize + 'static>(
     duration: Timestamp,
 ) -> Result<Errno, WasiError> {
     wasi_try_ok!(WasiEnv::process_signals_and_exit(&mut ctx)?);
+
     if let Some(()) = unsafe { handle_rewind::<M, _>(&mut ctx) } {
         return Ok(Errno::Success);
     }
@@ -34,9 +37,10 @@ pub(crate) fn thread_sleep_internal<M: MemorySize + 'static>(
     if duration > 0 {
         let duration = Duration::from_nanos(duration);
         let tasks = env.tasks().clone();
-        __asyncify_with_deep_sleep::<M, _, _>(ctx, Duration::from_millis(50), async move {
-            tasks.sleep_now(duration).await;
-        })?;
+        let res =
+            __asyncify_with_deep_sleep::<M, _, _>(ctx, Duration::from_millis(50), async move {
+                tasks.sleep_now(duration).await;
+            })?;
     }
     Ok(Errno::Success)
 }
